///////////////////////////////////////////////////////////////////////////////////////////////
////                                                                                           
////   file name:    z80_sram.v                                                                             
////   description: simple static SRAM                                                                                    
////   project:     wb_z80
////
////
////   Author: B.J. Porcella                                                                   
////          bporcella@sbcglobal.net                                                          
////                                                                                           
////                                                                                           
////                                                                                           
///////////////////////////////////////////////////////////////////////////////////////////////
////                                                                                           
//// Copyright (C) 2000-2002 B.J. Porcella                                                     
////                         Real Time Solutions                                               
////                                                                                           
////                                                                                           
//// This source file may be used and distributed without                                      
//// restriction provided that this copyright statement is not                                 
//// removed from the file and that any derivative work contains                               
//// the original copyright notice and the associated disclaimer.                              
////                                                                                           
////     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY                                   
//// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED                                 
//// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS                                 
//// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR                                    
//// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,                                       
//// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES                                  
//// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE                                 
//// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR                                      
//// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF                                
//// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT                                
//// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                                
//// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE                                       
//// POSSIBILITY OF SUCH DAMAGE.                                                               
////                                                                                           
///////////////////////////////////////////////////////////////////////////////////////////////
//
//  DESCRIPTION:
//  This file was intended to be a generic sram module  -- and started out as generic_spram.v
//  However,  the generic_spram.v device contained not only an address register, but also a 
//  data register.   I guess in retrospect that I could design around that ----   by deleting the
//  address register of the z80 and also the data output register of the z80 and use the 
//  registers of the "generic_spram" for those functions.   
//
//  I have opted to hack my own model of a register array.  
//  (which I know can be reasonably synthesized in most technologies).   
//  Accordingly, I decided to re-name the file   -- it is very different 
//  in its behavior  --  there should be no mis-understanding here.
//
//  If this actually causes synthesis problems, please let me know.  I will try to help.
//   bj
//  
//
//
//  CVS Log
//
//  $Id: z80_sram.v,v 1.1 2004/05/27 14:28:55 bporcella Exp $
//
//  $Date: 2004/05/27 14:28:55 $
//  $Revision: 1.1 $
//  $Author: bporcella $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//      $Log: z80_sram.v,v $
//      Revision 1.1  2004/05/27 14:28:55  bporcella
//      Instruction test (with interrupts) runs!!!
//
//      Revision 1.1.1.1  2004/04/13 23:47:42  bporcella
//      import first files
//
//
//
//-------1---------2---------3--------Module Name and Port List------7---------8---------9--------0

module z80_sram(
    // Generic synchronous single-port RAM interface
    clk, rst, ce, we, oe, addr, di, do
);

    //
    // Default address and data buses width
    //
    parameter aw = 15; //number of address-bits
    parameter dw = 8; //number of data-bits

    //
    // Generic synchronous single-port RAM interface
    //


//-------1---------2---------3--------Output Ports---------6---------7---------8---------9--------0
    output [dw-1:0] do;   // output data bus

//-------1---------2---------3--------Input Ports----------6---------7---------8---------9--------0
    input           clk;  // Clock, rising edge
    input           rst;  // Reset, active high
    input           ce;   // Chip enable input, active high
    input           we;   // Write enable input, active high
    input           oe;   // Output enable input, active high
    input  [aw-1:0] addr; // address bus inputs
    input  [dw-1:0] di;   // input data bus
//-------1---------2---------3--------Parameters-----------6---------7---------8---------9--------0
//-------1---------2---------3--------Wires------5---------6---------7---------8---------9--------0
//-------1---------2---------3--------Registers--5---------6---------7---------8---------9--------0
//-------1---------2---------3--------Assignments----------6---------7---------8---------9--------0
//-------1---------2---------3--------State Machines-------6---------7---------8---------9--------0


reg  [dw-1:0] mem [(1<<aw)-1:0];    // RAM content

// bjp  change  was
//reg  [aw-1:0] raddr;             // RAM read address
//wire raddr = addr;
//
// Data output drivers
//
assign do =  mem[addr];



// write operation
always@(posedge clk)
    if (ce && we)
        mem[addr] <=  di;




//Nov.11.2004 TAK
	parameter init_file       = "z80_test2.ihx";
  reg [8*256 : 1] ram_initf;
	 initial begin
	  $convert_hex2ver(init_file, 8, ram_initf);
        $readmemh(ram_initf, mem);
	 $display("Initial Program Loaded. %s",ram_initf);//Jul.4.2005
	 $display("Several Portions are modified from original. Grep and see TAK.");
	$display(" RAM 32k->64K ");				
	 end


//Nov.9.2004 TAK
      `define Print_Port_Address 16'h83e8  //got z80_test.map file
      `define Print_CAHR_Port_Address 16'h83e9
      `define Print_INT_Port_Address 16'h83eb  // LAST ADDRESS
      `define Print_LONG_Port_Address 16'h83ef  // LAST ADDRESS
 reg [0:639] char_buffer;
	task Cprint;// String OUT until the byte 00 or xx detected with least Byte first and justified.
		integer i;
		begin :Block
			i=0;
			while (1) begin
				if (char_buffer[i*8 +:8] ===8'h00 || char_buffer[i*8 +:8]===8'hxx) begin
						disable Block;
				end	
				$write("%c",char_buffer[i*8 +:8]);
				i=i+1;  
			end
		end
	endtask

  
   integer  counter=0;    
   always @ (posedge clk ) begin
	 if (ce && we) begin 
             //  mem[$unsigned(a)] <= wr_data;
                     //	if (a >=0 && a<10000 && a !=`Print_Port_Address) $display("%h %h",a,wr_data);
           	   if (addr==`Print_Port_Address) begin
				if (di===8'h00) begin
					
				      char_buffer[counter*8 +:8]=di;
					if (char_buffer[0  +:8*7]==="$finish") begin
							$finish;				
							
					end else if (char_buffer[0  +:8*5]==="$time") begin
							$display("Current Time on Simulation=%d",$time);				
							
					end else  Cprint;//$write("%s",char_buffer);

					for (counter=0; counter< 80; counter=counter+1) begin
						char_buffer[counter*8 +:8]=8'h00;
					end
					counter=0;
				end else begin
					char_buffer[counter*8 +:8]=di;
					counter=counter+1;
				end
		   end
       	   else if (addr==`Print_CAHR_Port_Address) begin
				$write("%h ",di);
		   end else if (addr==`Print_INT_Port_Address) begin
				 $write("%h ",{di,mem[addr-1]});
		   end else if (addr==`Print_LONG_Port_Address) begin
				 $write("%h ",{di,mem[addr-1],mem[addr-2],mem[addr-3]});
		   end 
	end
   end //always

endmodule
